const { pool } = require("../config/database/database.config");

/*
Class responsible for interacting with the users table
and making queries to it.
*/

/*
Find user info by their username.
*/
async function findByUsername(username) {
  try {
    console.log("Finding user:", username);

    // include permission level in response
    const [results] = await pool.query(
      `SELECT users.*, rolesPermissions.permissionLevel
       FROM users
       JOIN rolesPermissions ON users.roleID = rolesPermissions.roleID
       WHERE users.username = ?`,
      [username]
    );

    console.log("Query completed. Results:", results);

    // Return the first result if found, otherwise return null
    return results.length > 0 ? results[0] : null;
  } catch (err) {
    console.error("Database error in findByUsername:", err);
    throw err; // Propagate the error
  }
}

/*
Function to add or update a user in the repository
*/
async function upsertUser(username, roleID, roleInfo, peopleId) {
  try {
    const existingUser = await findByUsername(username);

    if (existingUser) {
      // User exists, so update roleID and roleInfo
      console.log("Updating existing user:", username);

      const [updateResult] = await pool.query(
        "UPDATE users SET roleID = ?, roleInfo = ? WHERE username = ?",
        [roleID, roleInfo, username]
      );

      console.log("User updated successfully:", updateResult);
      return {
        message: "User updated successfully",
        updated: true,
        insertId: updateResult.insertId,
      };
    } else {
      // User does not exist, so insert a new user
      console.log("Creating new user:", username);

      const [insertResult] = await pool.query(
        "INSERT INTO users (username, roleID, roleInfo, peopleId) VALUES (?, ?, ?,?)",
        [username, roleID, roleInfo, peopleId]
      );

      console.log("User created successfully:", insertResult);
      return {
        message: "User created successfully",
        created: true,
        insertId: insertResult.insertId,
      };
    }
  } catch (error) {
    console.error("Error in upsertUser:", error);
    throw error;
  }
}

module.exports = { findByUsername, upsertUser };
