const express = require("express");
const {
  viewLoginsTimerange,
  viewDatabaseChangesTimerange,
} = require("../service/activityLoggerService");
const { authenticateAdminOnly } = require("../config/auth/authenticateJWT");

const activityRouter = express.Router();

// get login history by timerange
activityRouter.get("/login-data", authenticateAdminOnly, async (req, res) => {
  const { start, end } = req.query;

  // ensure both params are given
  if (!start || !end) {
    return res
      .status(400)
      .json({ message: "Please provide both start and end dates." });
  }

  try {
    const data = await viewLoginsTimerange(start, end);
    res.status(200).json(data);
  } catch (error) {
    console.log("error fetching login data: ", error);
    res.status(500).json({ message: "Internal service error." });
  }
});

activityRouter.get("/db-changes", async (req, res) => {
  const { start, end } = req.query;

  if (!start || !end) {
    return res.status(400).json({ message: "start and end are required." });
  }

  try {
    const changes = await viewDatabaseChangesTimerange(start, end);
    return res.status(200).json(changes);
  } catch (error) {
    console.error("Error fetching database changes:", error);
    return res
      .status(500)
      .json({ message: "Error fetching database changes." });
  }
});

// get db history for timerange

module.exports = activityRouter;
