const {
  logSuccessfulLogin,
  getLoginDataTimerange,
  logDatabaseChange,
  getDatabaseChangeTimerange,
} = require("../repository/activityRepository");

/*
Function that creates a row of data in the userLogin table 
to documenmt a successful log in.
*/
async function documentSuccessfulLogin(userId, timestamp) {
  try {
    const result = await logSuccessfulLogin(userId, timestamp);
  } catch (error) {
    // SQL error from the repository
    console.log("Error when logging a successful login: ", error);
  }
}

async function documentDatabaseChange(
  tableName,
  recordId,
  userId,
  changeType,
  changedColumns,
  oldValues,
  newValues
) {
  try {
    const changeLogId = await logDatabaseChange(
      tableName,
      recordId,
      userId,
      changeType,
      changedColumns,
      oldValues,
      newValues
    );
    console.log(`Change logged with ID: ${changeLogId}`);
  } catch (error) {
    console.error("Error documenting database change:", error);
  }
}

/*
Get the login data for a sopecific time range
*/
async function viewLoginsTimerange(start, end) {
  // will throw error if there is a problem
  return await getLoginDataTimerange(start, end);
}

async function viewDatabaseChangesTimerange(startTime, endTime) {
  try {
    const changes = await getDatabaseChangeTimerange(startTime, endTime);

    console.log("changes: ", changes);
    // Post-process the changes
    return changes.map((change) => ({
      logId: change.logId,
      changedTable: change.changedTable,
      recordId: change.recordId,
      userId: change.userId,
      changeType: change.changeType,
      changedColumns: Array.isArray(change.changedColumns)
        ? change.changedColumns //
        : JSON.parse(change.changedColumns),
      oldValues:
        typeof change.oldValues === "string"
          ? JSON.parse(change.oldValues)
          : change.oldValues,
      newValues:
        typeof change.newValues === "string"
          ? JSON.parse(change.newValues)
          : change.newValues,
      changeTimestamp: new Date(change.changeTimestamp).toISOString(),
    }));
  } catch (error) {
    console.error("Error viewing database changes:", error);
    throw error;
  }
}

module.exports = {
  documentSuccessfulLogin,
  viewLoginsTimerange,
  documentDatabaseChange,
  viewDatabaseChangesTimerange,
};
