const {
  addAddress,
  getAddressesForPerson,
  deleteAddressById,
  updateAddress,
} = require("../repository/addressRepository");
const {
  addContact,
  getContactsForPerson,
  updateContact,
  deleteContactById,
} = require("../repository/contactRepository");
const {
  addDegree,
  getDegreeIdsForPerson,
  deleteDegreeById,
  updateDegree,
  getDegreesForPerson,
} = require("../repository/degreeRepository");
const {
  addPerson,
  searchPeopleByName,
  deletePersonById,
  updatePerson,
  getPersonById,
  search,
} = require("../repository/peopleRepository");
const {
  addPeopleInvolvement,
  getInvolvementType,
  getInvolvement,
  getInvolvementsForPerson,
  deleteInvolvementForPerson,
  getInvolvementTypeById,
} = require("../repository/involvementRepository");
const { documentDatabaseChange } = require("./activityLoggerService");
const { upsertUser } = require("../repository/userRepository");
const { getStateId } = require("../repository/stateRepository");
const { getChapterById } = require("../repository/chapterRepository");
const xlsx = require("xlsx");

/*
Function to handle the logic for reading in a file and creating all people listed in file.
*/
async function readFile(file) {
  try {
    const myFile = xlsx.readFile(file); // Read the file
    // Loop through all sheets
    myFile.SheetNames.forEach((sheetName) => {
      const worksheet = myFile.Sheets[sheetName];
      // Convert the sheet data to JSON
      let jsonData = xlsx.utils.sheet_to_json(worksheet);

      // Trim headers and convert to lowercase
      jsonData = jsonData.map((row) => {
        const newRow = {};
        for (const key in row) {
          // Trim the key and convert it to lowercase
          const trimmedKey = key.trim().toLowerCase();
          newRow[trimmedKey] = row[key]; // Keep the original value
        }
        // console.log(newRow);
        return newRow;
      });
      jsonData.forEach((row) => {
        // Access data using header names
        const preferredYr = row["preferred class year"]; // Will the headers always be the same?? -- leading and trailing space and caps dont matter
        const id = row["id"];
        const notes = row["notes"];
        const lastName = row["last name"];
        const firstName = row["first name"];
        const nickname = row["nickname"];
        const maidenName = row["maiden name"];
        const middleName = row["middle name"];
        const gender = row["gender"];
        const address = row["address"];
        const address2 = row["address 2"];
        const zip = row["zip"];
        const city = row["city"];
        const state = row["state"];
        const chapter = row["chapter"];
        const phone = row["phone"];
        const phone2 = row["phone 2"];
        const doesNotGetEmail = row["does not receive alumni email"];
        const email = row["email"];
        const email2 = row["email 2"];
        const email3 = row["email 3"];
        const email4 = row["email 4"];
        const preferredCollege = row["preferred college"];
        const degree1 = row["degree 1"];
        const department1 = row["department 1"];
        const college1 = row["college 1"];
        const degree2 = row["degree 2"];
        const department2 = row["department 2"];
        const college2 = row["college 2"];
        const involvements = {};

        for (let i = 0; i < 15; i++) {
          let num = i + 1;
          let value = "involvement " + num;
          involvements[i] = row[value];
        }
        console.log(`PreferredYr: ${preferredYr}, lastname: ${lastName}`); //to test

        //now create new person with these values and call createPerson(person)

        const person = {
          personInfo: {
            firstName: firstName, // Default to empty string if cnot available
            lastName: lastName,
            middleName: middleName || null,
            maidenName: maidenName || null,
            suffix: null, // Default to empty string
            nickName: nickname || null,
            techAlumniChapter: chapter || null, // Default if not provided
            classYear: preferredYr, // Default to null if not provided
            gradYear: null,
            gradSemester: null,
            gender: gender || null,
          },
          degrees: [
            {
              degreeType: degree1 || null,
              degreeYear: preferredYr || null,
              degreeDescription: null,
              degreeDepartment: department1 || null,
              degreeCollege: college1 || null,
            },
          ],
          addresses: [
            {
              address1: address || null,
              address2: address2 || null,
              city: city || null,
              state: state || null,
              country: "USA", // Default to "USA" ??
              zipCode: zip || null,
              preferredAddress: true, // Assuming it is preferred by default ??
            },
          ],
          contacts: [
            {
              contactNumber: phone || "",
              contactType: "phone", // Assuming 'phone' for all contact types ??
              preferredContact: true, // Assuming this contact is preferred by default ??
            },
          ],
          involvements: [
            involvements[0] || "",
            involvements[1] || "",
            involvements[2] || "",
            involvements[3] || "",
            involvements[4] || "",
            involvements[5] || "",
            involvements[6] || "",
            involvements[7] || "",
            involvements[8] || "",
            involvements[9] || "",
            involvements[10] || "",
            involvements[11] || "",
            involvements[12] || "",
            involvements[13] || "",
            involvements[14] || "",
          ].filter((value) => value !== ""), // Remove empty strings if no involvements
          userInfo: null, // Default to null as required
        };
        // console.log(person);

        const myid = createPerson(person);
        console.log(myid);
        console.log("after");
      });
    });
  } catch (error) {
    console.log("ERROR");
    console.error(`Error reading file: ${error.message}`);
  }
}

/*
Function to handle the logic for creating a person.
*/
async function createPerson(person, userId) {
  // extract the parts of a person
  const { personInfo, degrees, addresses, contacts, involvements, userInfo } =
    person;

  console.log("Adding person....");
  // insert the personInfo into people table
  const newPersonID = await addPerson(personInfo);
  console.log("Added person ID: ", newPersonID);

  // document the database change
  await documentDatabaseChange(
    "people",
    newPersonID,
    userId,
    "INSERT",
    Object.keys(personInfo),
    null,
    personInfo
  );

  // insert the degree information into the database
  for (const degree of degrees) {
    const addedDegree = await addDegree(degree, newPersonID);
    console.log("added degree: ", addedDegree);

    // document the database change
    await documentDatabaseChange(
      "degrees",
      addedDegree,
      userId,
      "INSERT",
      Object.keys(degree),
      null,
      degree
    );
  }

  // insert the address information into the database
  for (const address of addresses) {
    const addedAddress = await addAddress(address, newPersonID);
    console.log("added address: ", addedAddress);

    // document the database change
    await documentDatabaseChange(
      "addresses",
      addedAddress,
      userId,
      "INSERT",
      Object.keys(address),
      null,
      address
    );
  }

  // insert the contact information into the database
  for (const contact of contacts) {
    const addedContact = await addContact(contact, newPersonID);
    console.log("added contact: ", addedContact);

    // document the database change
    await documentDatabaseChange(
      "contacts",
      addedContact,
      userId,
      "INSERT",
      Object.keys(contact),
      null,
      contact
    );
  }

  // insert the involvement information into the database
  for (const involvement of involvements) {
    // retrieve involvement Type
    const involvementType = await getInvolvementType(involvement);

    if (involvementType) {
      // get the invovelment id and link to person
      const involvement = await getInvolvement(
        involvementType.involvementTypeId
      );
      if (involvement) {
        const addedInvolvementId = await addPeopleInvolvement(
          newPersonID,
          involvement.involvementId
        );

        // Log the database change for involvement
        await documentDatabaseChange(
          "peopleXInvolvement",
          addedInvolvementId,
          userId,
          "INSERT",
          ["peopleId", "involvementId"],
          null,
          {
            peopleId: newPersonID,
            involvementId: involvement.involvementId,
          }
        );
      } else {
        console.log("invovlement not found");
      }
    } else {
      console.log("involvement does not exist: ", involvement);
    }
  }

  // add the user info
  if (userInfo) {
    // get the pid
    const pid = userInfo.pid;
    if (pid) {
      try {
        const newUserId = await upsertUser(
          userInfo.pid,
          userInfo.roleId,
          userInfo.info,
          newPersonID
        );

        console.log("New user id: ", newUserId);

        await documentDatabaseChange(
          "users",
          newUserId.insertId,
          userId,
          "INSERT",
          ["username", "roleID", "roleInfo", "peopleId"],
          null,
          {
            username: userInfo.pid,
            roleID: userInfo.roleId,
            roleInfo: userInfo.info,
            peopleId: newPersonID,
          }
        );
      } catch (error) {
        console.log("error upserting user.");
      }
    } else {
      console.log("No pid found");
    }
  }
  console.log("person created!");

  return newPersonID;
}

/*
Function to call repository and get people with a 
gicen first and last name.
*/
async function findByName(firstName, lastName) {
  // call repository function, null will be returned if no people are found
  const peopleFound = await searchPeopleByName(firstName, lastName);
  return peopleFound;
}

/*
Function to delete person by their id.
*/
async function deleteById(peopleId, userId) {
  // delete the person
  const personToDelete = await getPersonFullDetails(peopleId);

  if (!personToDelete) {
    console.log(`Person with ID ${peopleId} not found.`);
    return false;
  }

  // Delete the person
  const personDeleted = await deletePersonById(peopleId);

  if (personDeleted) {
    // Log the database change
    await documentDatabaseChange(
      "people",
      peopleId,
      userId,
      "DELETE",
      Object.keys(personToDelete.personInfo),
      personToDelete.personInfo,
      null
    );

    console.log(`Deleted person with ID ${peopleId}`);
    return true;
  }
  return true;
}

/*
Function to update all of a person with the given id's information.
*/
async function updatePersonById(personId, person, userId) {
  console.log("Person: ", person);

  // Get the current details of the person before the update
  const oldPersonDetails = await getPersonFullDetails(personId);
  if (!oldPersonDetails) {
    console.log(`Person with ID ${personId} not found.`);
    return null;
  }

  const { personInfo, degrees, addresses, contacts, involvements } = person;

  console.log("Updating person...");
  console.log("Person info: ", personInfo);

  // update personInfo
  const updatedPersonInfo = await updatePerson(personId, personInfo);
  if (updatedPersonInfo == false) {
    return null;
  }

  // Log the database change for personInfo
  await documentDatabaseChange(
    "people",
    personId,
    userId,
    "UPDATE",
    Object.keys(personInfo),
    oldPersonDetails.personInfo,
    personInfo
  );

  // update degrees
  if (degrees) {
    await updateDegreesForPerson(personId, degrees, userId);
  }

  // update address info
  if (addresses) {
    await updateAddressesForPerson(personId, addresses, userId);
  }

  // update contact info
  if (contacts) {
    await updateContactsForPerson(personId, contacts, userId);
  }

  await updateInvolvementsForPerson(personId, involvements, userId);

  return true;
}

/*
Function to get all the data that is associated with a single 
person.
*/
async function getPersonFullDetails(personId) {
  // get the personal details
  const personInfo = await getPersonById(personId);

  // get the degree information
  const degrees = await getDegreesForPerson(personId);

  // get the address information
  const addresses = await getAddressesForPerson(personId);

  // get the contact information
  const contacts = await getContactsForPerson(personId);

  // get the involvement information (none for now)
  const involvements = await getInvolvementsForPerson(personId);

  // return all of the data
  return {
    personInfo,
    degrees,
    addresses,
    contacts,
    involvements,
  };
}

/*
Helper function to update the degree information for a person.
*/
async function updateDegreesForPerson(personId, degrees, userId) {
  // Get the person's current degrees
  const currentDegreeIds = await getDegreeIdsForPerson(personId);
  let updatedDegreeIds = [];

  // Add/update the given degrees
  for (const degree of degrees) {
    const peopleDegreeId = degree.peopleDegreeId || null;

    // degree does not previously have a peopleDegreeId -> this means its new (add degree)
    if (!peopleDegreeId) {
      const addedDegree = await addDegree(degree, personId);
      // Log the database change for added degree
      await documentDatabaseChange(
        "degrees",
        addedDegree,
        userId,
        "INSERT",
        Object.keys(degree),
        null,
        degree
      );
      console.log("Added new degree: ", addedDegree);
    } else {
      console.log("in service updating degree now");
      // previous peopleDegreeId exists, update the degree
      const updatedDegree = await updateDegree(
        peopleDegreeId,
        personId,
        degree
      );

      console.log("Updated degree: ", updatedDegree);

      await documentDatabaseChange(
        "degrees",
        updatedDegree.newValue.peopleDegreeId,
        userId,
        "UPDATE",
        Object.keys(degree),
        updatedDegree.oldValue,
        updatedDegree.newValue
      );

      console.log("Updated degree: ", updatedDegree);
      if (updatedDegree) {
        updatedDegreeIds.push(peopleDegreeId);
      }
    }
  }

  // Delete degrees not included in the update
  for (const degreeId of currentDegreeIds) {
    if (!updatedDegreeIds.includes(degreeId)) {
      const result = await deleteDegreeById(degreeId);

      // Log the database change for deleted degree
      await documentDatabaseChange(
        "degrees",
        degreeId,
        userId,
        "DELETE",
        Object.keys(result),
        result,
        null
      );
      console.log("Deleted degree: ", degreeId);
    }
  }
}

/*
Helper function to update addresses for a person.
*/
async function updateAddressesForPerson(personId, addresses, userId) {
  // Get the current addresses
  const currentAddresses = await getAddressesForPerson(personId);
  let updatedAddressIds = [];

  for (const address of addresses) {
    const addressId = address.addressId || null;

    if (!addressId) {
      const addedAddress = await addAddress(address, personId);

      if (addedAddress) {
        // Log the database change for added address
        await documentDatabaseChange(
          "addresses",
          addedAddress,
          userId,
          "INSERT",
          Object.keys(address),
          null,
          address
        );
        console.log("Added new address: ", addedAddress);
      } else {
        console.log("added address not found");
      }
    } else {
      const updatedAddress = await updateAddress(addressId, personId, address);
      console.log("Updated address: ", updateAddress);

      if (updatedAddress != null) {
        console.log("Updated address: ", updatedAddress);

        // Log the database change for updated address
        await documentDatabaseChange(
          "addresses",
          addressId,
          userId,
          "UPDATE",
          Object.keys(address),
          updatedAddress.oldValue,
          updatedAddress.newValue
        );

        updatedAddressIds.push(addressId);
      } else {
        console.log("Error updating adddress");
      }
    }
  }

  // Delete addresses not included in the update
  for (const currentAddress of currentAddresses) {
    if (!updatedAddressIds.includes(currentAddress.addressId)) {
      await deleteAddressById(currentAddress.addressId);
      console.log("Deleted address: ", currentAddress.addressId);
    }
  }
}

/*
Helper function to update contacts for a person.
*/
async function updateContactsForPerson(personId, contacts, userId) {
  // Get the current contacts for the person
  const currentContacts = await getContactsForPerson(personId);
  const updatedContactIds = [];

  for (const contact of contacts) {
    const contactId = contact.peopleContactId || null;

    if (!contactId) {
      // Add a new contact
      const addedContact = await addContact(contact, personId);

      if (addedContact) {
        // Log the database change for added contact
        await documentDatabaseChange(
          "peopleContact",
          addedContact,
          userId,
          "INSERT",
          Object.keys(contact),
          null,
          contact
        );
        console.log("Added new contact: ", addedContact);
      }
    } else {
      // Update an existing contact
      const updatedContact = await updateContact(contactId, personId, contact);

      if (updatedContact) {
        const { oldValue, newValue } = updatedContact;
        console.log("Updated contact: ", updatedContact);

        // Log the database change for updated contact
        await documentDatabaseChange(
          "peopleContact",
          contactId,
          userId,
          "UPDATE",
          Object.keys(contact),
          oldValue,
          newValue
        );

        updatedContactIds.push(contactId);
      } else {
        console.log("Error updating contact.");
      }
    }
  }

  // Delete contacts that are not included in the update
  for (const currentContact of currentContacts) {
    if (!updatedContactIds.includes(currentContact.peopleContactId)) {
      const deletedContact = await deleteContactById(
        currentContact.peopleContactId
      );

      if (deletedContact) {
        // Log the database change for deleted contact
        await documentDatabaseChange(
          "peopleContact",
          currentContact.peopleContactId,
          userId,
          "DELETE",
          Object.keys(currentContact),
          currentContact,
          null
        );

        console.log("Deleted contact: ", currentContact.peopleContactId);
      }
    }
  }
}

async function updateInvolvementsForPerson(personId, involevements, userId) {
  // Step 1: Retrieve the current involvements for the person
  const currentInvolvements = await getInvolvementsForPerson(personId);

  console.log("current involvements: ", currentInvolvements);

  const newInvolvementIds = [];
  for (const involvement of involevements) {
    // get the involvement type for each invovlement
    const involvementType = await getInvolvementType(involvement);

    if (involvementType) {
      // get the invovelment id and link to person
      const involvement = await getInvolvement(
        involvementType.involvementTypeId
      );
      if (involvement) {
        console.log("invovlement id: ", involvement.involvementId);
        newInvolvementIds.push(involvement.involvementId);

        if (!currentInvolvements.includes(involvement)) {
          const addedInvolvementId = await addPeopleInvolvement(
            personId,
            involvement.involvementId
          );

          // Log the database change for added involvement
          await documentDatabaseChange(
            "peopleXInvolvement",
            addedInvolvementId,
            userId,
            "INSERT",
            ["peopleId", "involvementId"],
            null,
            { peopleId: personId, involvementId: involvement.involvementId }
          );
        } else {
          console.log("involvement already here!");
        }
      } else {
        console.log("invovlement not found");
      }
    }
  }

  // Step 2: Remove involvements that are no longer in the input list
  for (const currentInvolvement of currentInvolvements) {
    const involvementType = await getInvolvementType(currentInvolvement);
    if (involvementType) {
      const involvement = await getInvolvement(
        involvementType.involvementTypeId
      );
      if (involvement) {
        if (!newInvolvementIds.includes(involvement.involvementId)) {
          const deleted = await deleteInvolvementForPerson(
            personId,
            involvement.involvementId
          );
          console.log("Removing involvement: ", involvement.involvementId);
          if (deleted) {
            // Log the database change for deleted involvement
            await documentDatabaseChange(
              "peopleXInvolvement",
              involvement.involvementId,
              userId,
              "DELETE",
              ["peopleId", "involvementId"],
              { peopleId: personId, involvementId: involvement.involvementId },
              null
            );

            console.log(
              `Removed involvement ${currentInvolvement} for person ${personId}`
            );
          }
        }
      } else {
        console.log("invovlement not found: ", currentInvolvement);
      }
    } else {
      console.log("invovlement not found: ", currentInvolvement);
    }
  }
}

async function generalSearch(filters) {
  // combine corpsUnits and corpsOrgs lists into one list of involvements
  const involvementNames = [
    ...(filters.corpsUnits || []),
    ...(filters.corpsOrgs || []),
  ].filter((name) => name && name.trim() !== "");

  // find the involvement type for each involvement
  let involvementTypes = [];
  try {
    involvementTypes = await Promise.all(
      involvementNames.map(async (name) => {
        try {
          const involvementType = await getInvolvementType(name); // Fetch the full involvement type object
          return involvementType.involvementTypeId; // Extract and return only the ID
        } catch (error) {
          console.error(`Error getting involvement type for: ${name}`, error);
          return null; // Return null for failed translations
        }
      })
    );
  } catch (error) {
    console.error("Error processing involvement types:", error);
  }

  // Filter out nulls in case of translation failures
  involvementTypes = involvementTypes.filter((type) => type !== null);

  // translate state into a state id
  let stateId = null;
  if (filters.state) {
    try {
      stateId = await getStateId(filters.state);
    } catch (error) {
      console.log("error getting state id");
    }
  }

  filters.stateId = stateId;
  filters.involvementType = involvementTypes;

  let results = await search(filters);
  console.log(results);

  // post processing:
  for (let result of results) {
    // 1. translate tech alumni chapter
    if (result.techAlumniChapter) {
      try {
        const chapterName = await getChapterById(result.techAlumniChapter);
        result.techAlumniChapter = chapterName;
      } catch (error) {
        console.error(
          `Error translating techAlumniChapter: ${result.techAlumniChapter}`,
          error
        );
      }
    }

    // 2. separate degrees
    if (result.degrees) {
      result.degrees = result.degrees.split(" | ").map((degree) => {
        const [year, details] = degree.split(" - ");
        const [department, college] = details.slice(0, -1).split(" (");
        return {
          degreeYear: year.trim(),
          degreeDepartment: department.trim(),
          degreeCollege: college.trim(),
        };
      });
    }

    // 3. separate addresses
    if (result.addresses) {
      result.addresses = result.addresses.split(" | ").map((address) => {
        const [city, state] = address.split(", ");
        return {
          city: city.trim(),
          state: state.trim(),
        };
      });
    }

    // 4. separate contacts
    if (result.contactNumbers) {
      result.contactNumbers = result.contactNumbers
        .split(", ")
        .map((contact) => contact.trim());
    }

    // 5. separate involvements
    if (result.involvements) {
      result.involvements = result.involvements
        .split(", ")
        .map((involvement) => parseInt(involvement, 10));
    }

    // 6. Translate involvements
    if (result.involvements) {
      result.involvements = await Promise.all(
        result.involvements.map(async (involvementId) => {
          try {
            const involvementType = await getInvolvementTypeById(involvementId);
            const involvementName = involvementType.description;
            console.log("Name: ", involvementName);
            return {
              involvementName,
            };
          } catch (error) {
            console.error(
              `Error translating involvementId: ${involvementId}`,
              error
            );
            return { involvementId, involvementName: "Unknown" };
          }
        })
      );
    }
  }

  return results;
}

module.exports = {
  readFile,
  createPerson,
  findByName,
  deleteById,
  updatePersonById,
  getPersonFullDetails,
  generalSearch,
};
