import "./Login.css";
import user_icon from "../assets/person.png";
import React, { useState } from "react";
import { useNavigate } from "react-router-dom";
import axios from "axios";

function Login() {
  const [error, setError] = useState("");
  const navigate = useNavigate();

  // call backend CAS for login
  // const handleLogin = () => {
  //   const casLoginUrl = `${process.env.REACT_APP_API_URL}/login`;

  //   // Redirect the user to the CAS login page via backend
  //   window.location.href = casLoginUrl;
  // };

  const handleLogin = async () => {
    const casLoginUrl = `${process.env.REACT_APP_API_URL}/login`;
  
    try {
      // Make an HTTP request (GET or POST) to your backend to check if it's successful
      const response = await fetch(casLoginUrl, { method: 'GET' }); // or POST depending on your backend
  
      if (response.ok) {
        // If the response is OK, proceed to redirect to the CAS login page
        window.location.href = casLoginUrl;
      } else {
        // If the response is not OK (i.e., failed), you can show an error
        const errorData = await response.json();
        console.log("Error:", errorData.message);
        setError("User Not Found!\n Please try again or login as guest."); // Optionally set an error message in the state
      }
    } catch (err) {
      console.error("Network error or backend failure:", err);
      setError("Network error or backend failure");
    }
  };

  // redirect to home page without a jwt token
  const handleGuest = () => {
    navigate("/home");
  };

  // Close the error popup
  const closeErrorPopup = () => {
    setError(""); // Reset the error message
  };
  

  return (
    <div className="container">
      <div className="header">
        <div className="text">VT Corps Directory Log In Page</div>
        <div className="underline"></div>
      </div>
      {/* Conditionally render error message */}
      {error && (
        <div className="error-popup">
          <div className="error-message">
            {/* Split the error message into multiple lines */}
            {error.split("\n").map((line, index) => (
              <React.Fragment key={index}>
                <div className={index === 0 ? "error-main" : "error-sub"}>{line}</div>
                {index === 0 && <br />} {/* Only insert <br /> after the first line */}
              </React.Fragment>
            ))}
            <button className="close-btn" onClick={closeErrorPopup}>
              &times; {/* This is the "X" close button */}
            </button>
          </div>
        </div>
      )}
      <div className="inputs">
        <div className="home-auth">
          <button className="login-button" onClick={handleLogin}>
            Login with CAS
          </button>
        </div>
        <div className="home-auth">
          <button className="guest-button" onClick={handleGuest}>
            Continue as Guest
          </button>
        </div>
      </div>
      {/* <div className="forgot-password">Forgot Password <span> Click here</span></div> */}
      <div className="submitTerms">
        I agree to terms of use
        <img src={user_icon} alt="" />
      </div>
    </div>
  );
}

export default Login;
