import React, { useState } from "react";
import axios from "axios";
import styles from "./SearchBrowse.module.css";

const SearchBrowse = () => {
  const [searchParams, setSearchParams] = useState({
    firstName: null,
    lastName: null,
    nickName: null,
    classYearStart: null,
    classYearEnd: null,
    gradYearStart: null,
    gradYearEnd: null,
    degreeDepartment: null,
    degreeCollege: null,
    city: null,
    state: null,
    corpsUnits: [],
    corpsOrgs: [],
  });

  const [results, setResults] = useState([]);
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState(null);

  const handleInputChange = (e) => {
    const { name, value } = e.target;

    if (name === "corpsUnits" || name === "corpsOrgs") {
      setSearchParams((prev) => ({
        ...prev,
        [name]: value
          .split(",")
          .map((item) => item)
          .filter((item) => item !== ""), // Split by commas and trim each item
      }));
    } else {
      setSearchParams((prev) => ({
        ...prev,
        [name]: value || null, // Allow spaces for other inputs
      }));
    }
  };

  const handleSearch = async () => {
    setLoading(true);
    setError(null);
    setResults([]); // clear previous results

    try {
      const response = await axios.post(
        `${process.env.REACT_APP_API_URL}/person/search`,
        { filters: searchParams },
        { headers: { "Content-Type": "application/json" } }
      );

      setResults(response.data.people || []);
    } catch (err) {
      setError(
        err.response?.data?.message || "An error occurred during the search."
      );
    } finally {
      setLoading(false);
    }
  };

  // const filterAndSortResults = (data, searchParams) => {
  //   return data
  //     .filter((person) => {
  //       return Object.keys(searchParams).every((key) => {
  //         const value = searchParams[key];
  //         if (!value) return true; // If no value is provided, allow all results

  //         // Handle year-related fields (classYear, gradYear, degreeYear)
  //         if (key === "classYearStart" || key === "classYearEnd") {
  //           const start = parseInt(searchParams.classYearStart || "0", 10);
  //           const end = parseInt(searchParams.classYearEnd || "9999", 10);
  //           return person.classYear >= start && person.classYear <= end;
  //         }

  //         if (key === "gradYearStart" || key === "gradYearEnd") {
  //           const start = parseInt(searchParams.gradYearStart || "0", 10);
  //           const end = parseInt(searchParams.gradYearEnd || "9999", 10);
  //           return person.gradYear >= start && person.gradYear <= end;
  //         }

  //         // if (key === "degreeYearStart" || key === "degreeYearEnd") {
  //         //   const start = parseInt(searchParams.degreeYearStart || "0", 10);
  //         //   const end = parseInt(searchParams.degreeYearEnd || "9999", 10);
  //         //   // Ensure degrees is an array and filter based on degreeYear
  //         //   return person.degrees?.some((degree) => {
  //         //     const degreeYear = parseInt(degree.degreeYear, 10);
  //         //     return degreeYear >= start && degreeYear <= end;
  //         //   });
  //         // }

  //         // Handle degree-related fields (degreeDepartment, degreeCollege)
  //         if (key === "degreeDepartment" || key === "degreeCollege") {
  //           return person.degrees?.some((degree) =>
  //             degree[key]?.toLowerCase().includes(value.toLowerCase())
  //           );
  //         }

  //         // Handle city and state fields (assuming addresses is an array of objects)
  //         if (key === "city" || key === "state") {
  //           return person.addresses?.some((address) =>
  //             address[key]?.toLowerCase().includes(value.toLowerCase())
  //           );
  //         }

  //         // Handle corpsUnit and corpsOrgs fields
  //         if (key === "corpsUnit") {
  //           return person.corpsUnits?.some((unit) =>
  //             unit.toLowerCase().includes(value.toLowerCase())
  //           );
  //         }

  //         if (key === "corpsOrgs") {
  //           return person.corpsOrgs?.some((org) =>
  //             org.toLowerCase().includes(value.toLowerCase())
  //           );
  //         }

  //         // For all other fields, compare the values as strings (case-insensitive)
  //         return person[key]?.toLowerCase().includes(value.toLowerCase());
  //       });
  //     })
  //     .sort((a, b) => {
  //       const relevanceA = calculateRelevance(a, searchParams);
  //       const relevanceB = calculateRelevance(b, searchParams);
  //       return relevanceB - relevanceA; // Sort by relevance in descending order
  //     });
  // };

  // const calculateRelevance = (person, searchParams) => {
  //   let relevance = 0;
  //   Object.keys(searchParams).forEach((key) => {
  //     const value = searchParams[key];
  //     if (value && person[key]?.toLowerCase().includes(value.toLowerCase())) {
  //       relevance += 1;
  //     }
  //   });
  //   return relevance;
  // };

  const handleDetailView = (person) => {
    console.log("person: ", person);
    localStorage.setItem("selectedPerson", JSON.stringify(person));
    window.location.href = `/user/search-results/${person.peopleId}`;
  };

  return (
    <div className={styles.container}>
      <h2>Search for a Person</h2>
      <div className={styles.searchForm}>
        {/* Text Inputs */}
        {[
          "firstName",
          "lastName",
          "nickName",
          "classYearStart",
          "classYearEnd",
          "gradYearStart",
          "gradYearEnd",
          "degreeDepartment",
          "degreeCollege",
          "city",
          "state",
        ].map((field) => (
          <label key={field} className={styles.inputField}>
            {field
              .replace(/([A-Z])/g, " $1")
              .replace(/^./, (str) => str.toUpperCase())}
            :
            <input
              type="text"
              name={field}
              value={searchParams[field] || ""}
              onChange={handleInputChange}
              placeholder={`Enter ${field}`}
            />
          </label>
        ))}

        {/* Comma-Separated Array Inputs */}
        {["corpsUnits", "corpsOrgs"].map((field) => (
          <label key={field} className={styles.inputField}>
            {field
              .replace(/([A-Z])/g, " $1")
              .replace(/^./, (str) => str.toUpperCase())}
            (comma-separated):
            <input
              type="text"
              name={field}
              value={(searchParams[field] || []).join(", ")} // Join array into a string
              onChange={handleInputChange}
              placeholder={`Enter ${field}, separated by commas`}
            />
          </label>
        ))}

        <button onClick={handleSearch} disabled={loading}>
          {loading ? "Searching..." : "Search"}
        </button>
      </div>

      {/* Results */}
      {error && <p className={styles.error}>{error}</p>}
      {!error && results.length === 0 && !loading && <p>No results found.</p>}
      <ul className={styles.resultsList}>
        {results.map((person) => (
          <li
            key={person.peopleId}
            className={styles.resultItem}
            onClick={() => handleDetailView(person)}
          >
            {person.firstName} {person.lastName}
          </li>
        ))}
      </ul>
    </div>
  );
};

export default SearchBrowse;
